"""
This type stub file was generated by pyright.
"""

from connexion.apps.asynchronous import AsyncApi, AsyncApp
from connexion.apps.flask import FlaskApi, FlaskApp
from connexion.context import request
from connexion.middleware import ConnexionMiddleware

from .apps import AbstractApp
from .apps.asynchronous import AsyncApp
from .datastructures import NoContent
from .exceptions import ProblemException
from .problem import problem
from .resolver import Resolution, Resolver, RestyResolver
from .utils import not_installed_error

"""
Connexion is a framework that automagically handles HTTP requests based on OpenAPI Specification
(formerly known as Swagger Spec) of your API described in YAML format. Connexion allows you to
write an OpenAPI specification, then maps the endpoints to your Python functions; this makes it
unique, as many tools generate the specification based on your Python code. You can describe your
REST API in as much detail as you want; then Connexion guarantees that it will work as you
specified.
"""
App = ...
Api = ...

__all__ = (
    "AsyncApi",
    "AsyncApp",
    "FlaskApi",
    "FlaskApp",
    "request",
    "ConnexionMiddleware",
    "AbstractApp",
    "AsyncApp",
    "NoContent",
    "ProblemException",
    "problem",
    "Resolution",
    "Resolver",
    "RestyResolver",
    "not_installed_error",
    "App",
    "Api",
)
