"""
This type stub file was generated by pyright.
"""

from .main import ASGIDecorator, FlaskDecorator, StarletteDecorator, WSGIDecorator

"""
This module defines decorators which Connexion uses to wrap user provided view functions.
"""

__all__ = ("ASGIDecorator", "FlaskDecorator", "StarletteDecorator", "WSGIDecorator")
