"""
This type stub file was generated by pyright.
"""

class UserMixin:
    """
    This provides default implementations for the methods that Flask-Login
    expects user objects to have.
    """
    __hash__ = ...
    @property
    def is_active(self) -> bool:
        ...
    
    @property
    def is_authenticated(self) -> bool:
        ...
    
    @property
    def is_anonymous(self) -> bool:
        ...
    
    def get_id(self) -> str:
        ...
    
    def __eq__(self, other: object) -> bool:
        """
        Checks the equality of two `UserMixin` objects using `get_id`.
        """
        ...
    
    def __ne__(self, other: object) -> bool:
        """
        Checks the inequality of two `UserMixin` objects using `get_id`.
        """
        ...
    


class AnonymousUserMixin:
    """
    This is the default object for representing an anonymous user.
    """
    @property
    def is_authenticated(self) -> bool:
        ...
    
    @property
    def is_active(self) -> bool:
        ...
    
    @property
    def is_anonymous(self) -> bool:
        ...
    
    def get_id(self) -> None:
        ...
    


