"""
This type stub file was generated by pyright.
"""

import abc
import typing as t
from typing_extensions import Protocol

class Framework(Protocol):
    @staticmethod
    @abc.abstractmethod
    def is_framework_response(response: t.Any) -> bool:
        """Return True if provided response is a framework type"""
        ...
    
    @classmethod
    @abc.abstractmethod
    def connexion_to_framework_response(cls, response):
        """Cast ConnexionResponse to framework response class"""
        ...
    
    @classmethod
    @abc.abstractmethod
    def build_response(cls, data: t.Any, *, content_type: str = ..., headers: dict = ..., status_code: int = ...):
        ...
    
    @staticmethod
    @abc.abstractmethod
    def get_request(*args, **kwargs):
        """Return a framework request from the context."""
        ...
    
    @staticmethod
    @abc.abstractmethod
    def get_body(request):
        """Get body from a framework request"""
        ...
    


