"""
This type stub file was generated by pyright.
"""

import types
import typing as t

_ReturnType = t.TypeVar("_ReturnType")
MaybeAwaitable = t.Union[t.Awaitable[_ReturnType], _ReturnType]
Environ = t.Mapping[str, object]
_WriteCallable = t.Callable[[bytes], t.Any]
_ExcInfo = t.Tuple[type, BaseException, types.TracebackType]
_StartResponseCallable = t.Callable[[str, t.Sequence[t.Tuple[str, str]]], _WriteCallable,]
_StartResponseCallableWithExcInfo = t.Callable[[str, t.Sequence[t.Tuple[str, str]], t.Optional[_ExcInfo]], _WriteCallable,]
StartResponse = t.Union[_StartResponseCallable, _StartResponseCallableWithExcInfo]
ResponseStream = t.Iterable[bytes]
WSGIApp = t.Callable[[Environ, StartResponse], ResponseStream]
