"""
This type stub file was generated by pyright.
"""

NoContent = ...
class MediaTypeDict(dict):
    """
    A dictionary where keys can be either media types or media type ranges. When fetching a
    value from the dictionary, the provided key is checked against the ranges. The most specific
    key is chosen as prescribed by the OpenAPI spec, with `type/*` being preferred above
    `*/subtype`.
    """
    def __getitem__(self, item):
        ...
    
    def get(self, item, default=...): # -> None:
        ...
    
    def __contains__(self, item): # -> bool:
        ...
    


