"""
This type stub file was generated by pyright.
"""

from contextvars import ContextVar
from starlette.types import Receive, Scope
from connexion.operations import AbstractOperation

UNBOUND_MESSAGE = ...
_context: ContextVar[dict] = ...
context = ...
_operation: ContextVar[AbstractOperation] = ...
operation = ...
_receive: ContextVar[Receive] = ...
receive = ...
_scope: ContextVar[Scope] = ...
scope = ...
request = ...
