"""
This type stub file was generated by pyright.
"""

__title__ = ...
__description__ = ...
__url__ = ...
__version_info__ = ...
__version__ = ...
__author__ = ...
__author_email__ = ...
__maintainer__ = ...
__license__ = ...
__copyright__ = ...
