"""
This type stub file was generated by pyright.
"""

import flask_restful
import flask_restx
from injector import Scope, __version__ as injector_version
from flask.views import View

def test_injections(): # -> None:
    class View2(View):
        ...
    
    

def test_resets(): # -> None:
    class OurScope(Scope):
        ...
    
    

def test_memory_leak(): # -> None:
    ...

def test_doesnt_raise_deprecation_warning(): # -> None:
    ...

def test_jinja_env_globals_support_injection(): # -> None:
    ...

def test_error_handlers_support_injection(): # -> None:
    class CustomException(Exception):
        ...
    
    

def test_view_functions_arent_modified_globally(): # -> None:
    class MyView(View):
        ...
    
    

def test_view_args_and_class_args_are_passed_to_class_based_views(): # -> None:
    class MyView(View):
        ...
    
    

def test_flask_restful_integration_works(): # -> None:
    class HelloWorld(flask_restful.Resource):
        ...
    
    

def test_flask_restx_integration_works(): # -> None:
    class HelloWorld(flask_restx.Resource):
        ...
    
    

def test_request_scope_not_started_before_any_request_made_regression(): # -> None:
    ...

def test_noninstrospectable_hooks_dont_crash_everything(): # -> None:
    ...

def test_instance_methods(): # -> None:
    class HelloWorldService:
        ...
    
    
    class HelloWorld:
        ...
    
    

if injector_version >= '0.12':
    def test_forward_references_work(): # -> None:
        class X:
            ...
        
        
    
def test_request_scope_covers_teardown_request_handlers(): # -> None:
    ...

def test_request_scope_covers_blueprint_teardown_request_handlers(): # -> None:
    ...

