"""
This type stub file was generated by pyright.
"""

import abc
import typing as t
from connexion.frameworks.abstract import Framework

logger = ...
class BaseResponseDecorator:
    def __init__(self, *, framework: t.Type[Framework], jsonifier) -> None:
        ...
    
    @abc.abstractmethod
    def __call__(self, function: t.Callable) -> t.Callable:
        ...
    
    def build_framework_response(self, handler_response):
        ...
    


class SyncResponseDecorator(BaseResponseDecorator):
    def __call__(self, function: t.Callable) -> t.Callable:
        ...
    


class AsyncResponseDecorator(BaseResponseDecorator):
    def __call__(self, function: t.Callable) -> t.Callable:
        ...
    


class NoResponseDecorator(BaseResponseDecorator):
    """Dummy decorator to skip response serialization."""
    def __call__(self, function: t.Callable) -> t.Callable:
        ...
    


