"""
This type stub file was generated by pyright.
"""

COOKIE_NAME = ...
COOKIE_DURATION = ...
COOKIE_SECURE = ...
COOKIE_HTTPONLY = ...
COOKIE_SAMESITE = ...
LOGIN_MESSAGE = ...
LOGIN_MESSAGE_CATEGORY = ...
REFRESH_MESSAGE = ...
REFRESH_MESSAGE_CATEGORY = ...
ID_ATTRIBUTE = ...
AUTH_HEADER_NAME = ...
SESSION_KEYS = ...
EXEMPT_METHODS = ...
USE_SESSION_FOR_NEXT = ...
