"""
This type stub file was generated by pyright.
"""

_signals = ...
user_logged_in = ...
user_logged_out = ...
user_loaded_from_cookie = ...
_user_loaded_from_header = ...
user_loaded_from_request = ...
user_login_confirmed = ...
user_unauthorized = ...
user_needs_refresh = ...
user_accessed = ...
session_protected = ...
def __getattr__(name): # -> NamedSignal:
    ...

