"""
This type stub file was generated by pyright.
"""

from flask.testing import FlaskClient

class FlaskLoginClient(FlaskClient):
    """
    A Flask test client that knows how to log in users
    using the Flask-Login extension.
    """
    def __init__(self, *args, **kwargs) -> None:
        ...
    


