[build-system]

requires = ["setuptools>=42", "wheel"]

build-backend = "setuptools.build_meta"

[tool.setuptools]
include-package-data = true

[tool.setuptools.package-data]
"*" = ["*.toml", "*.yaml"]

[tool.setuptools.dynamic]
readme = { file = ["README.md"], content-type = "text/markdown" }

[project]
name = "{{application.module_name}}"
version = "0.0.1"
dynamic = ["readme"]

dependencies = [
    "Ligare.web",
{% if 'database' in module %}
    "Ligare.database"
{% endif %}
]

[tool.pyright]
exclude = [
    ".venv",
    ".pytest_cache",
    ".github",
    ".git",
    "typings",
    "**/node_modules",
    "**/__pycache__",
]
typeCheckingMode = "basic"
