"""
This type stub file was generated by pyright.
"""

from .implementation import FastAPIAppRequestInstrumentationConfigurator, FastAPIRequestAdapter, FastAPIResponseAdapter

def is_fastapi_present(): # -> bool:
    ...

if is_fastapi_present():
    ...
