"""
This type stub file was generated by pyright.
"""

import typing as t
from connexion.uri_parsing import AbstractURIParser
from connexion.validators import AbstractRequestBodyValidator

logger = ...
class FormDataValidator(AbstractRequestBodyValidator):
    """Request body validator for form content types."""
    def __init__(self, *, schema: dict, required=..., nullable=..., encoding: str, strict_validation: bool, uri_parser: t.Optional[AbstractURIParser] = ...) -> None:
        ...
    


class MultiPartFormDataValidator(FormDataValidator):
    ...


