"""
This type stub file was generated by pyright.
"""

from .abstract import AbstractApp

"""
This module defines Connexion applications. A Connexion App wraps a specific framework application
and exposes a standardized interface for users to create and configure their Connexion application.
"""
__all__ = ("AbstractApp",)
