"""
This type stub file was generated by pyright.
"""

from jsonschema import Draft4Validator
from connexion.validators import AbstractRequestBodyValidator, AbstractResponseBodyValidator

logger = ...
class JSONRequestBodyValidator(AbstractRequestBodyValidator):
    """Request body validator for json content types."""
    def __init__(self, *, schema: dict, required=..., nullable=..., encoding: str, strict_validation: bool, **kwargs) -> None:
        ...
    


class DefaultsJSONRequestBodyValidator(JSONRequestBodyValidator):
    """Request body validator for json content types which fills in default values. This Validator
    intercepts the body, makes changes to it, and replays it for the next ASGI application."""
    MUTABLE_VALIDATION = ...
    @staticmethod
    def extend_with_set_default(validator_class):
        ...
    


class JSONResponseBodyValidator(AbstractResponseBodyValidator):
    """Response body validator for json content types."""
    @property
    def validator(self) -> Draft4Validator:
        ...
    


class TextResponseBodyValidator(JSONResponseBodyValidator):
    ...


